{-# LANGUAGE FlexibleInstances, MultiParamTypeClasses, TypeFamilies,
             GeneralizedNewtypeDeriving, ScopedTypeVariables #-}
{-# OPTIONS_GHC -Wall -fno-warn-orphans -fno-warn-name-shadowing #-}
module Lam.LamBinding where

import Control.Applicative
import Control.Monad
import BoundLanguage
import Data.Maybe
import Lam.LamLanguage
import Lam.Lam

import qualified Data.Map as Map
import Data.Map (Map)


class LamBinder c where
  addBinding :: String -> c a -> c a
  lookupBinding :: Integer -> c (Maybe String)
  getBindingMap :: c (Map Integer String)

newtype LamBinderImpl a = LBI { fromLBI :: Map Integer String -> a }
  deriving (Functor, Applicative, Monad)

instance LamBinder LamBinderImpl where
  addBinding s (LBI f1) = (LBI f2) 
    where
      f2 = f1 . Map.insert 0 s . Map.mapKeysMonotonic succ

  lookupBinding i = LBI (Map.lookup i)

  getBindingMap = LBI id


instance (LamBinder c) => BoundLanguage Lam (c a) where
  bindingHook (CW ctx) hole = bindingHook' ctx
    where
      bindingHook' :: ContextI from to -> c a
      bindingHook' (ExpToAbs str _) = addBinding str hole
      bindingHook' _ = hole
